"""AuraMCP - Universal MCP Server Generator

Build MCP servers from any OpenAPI spec.
"""

__version__ = "0.1.1"

from auramcp.generator.parser import OpenAPIParser, ParsedSpec
from auramcp.generator.converter import MCPConverter, MCPTool
from auramcp.generator.codegen import MCPCodeGenerator, GeneratedServer, GeneratedFile

__all__ = [
    "OpenAPIParser",
    "ParsedSpec",
    "MCPConverter",
    "MCPTool",
    "MCPCodeGenerator",
    "GeneratedServer",
    "GeneratedFile",
    "__version__",
]
